﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using System.Configuration;
using System.Linq.Expressions;
using ApiLog.Act.Models.Log.Entity;

namespace ApiLog.Act.DAL
{
    public class MongoLogRepository
    {
        protected static IMongoClient _client;
        protected static MongoClientSettings _setting;
        protected static IMongoDatabase _database;

        #region 构造方法 测试环境
        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="isExpand">是否使用扩展的mongo，默认使用自己的mongo</param>
        public MongoLogRepository()
        {
            string name = "mongo_edc_log";
            string connStr = ConfigurationManager.ConnectionStrings[name].ConnectionString;
            MongoUrl url = new MongoUrl(connStr);
            if (_client == null)
                _client = new MongoClient(url);
            if (_database == null)
                _database = _client.GetDatabase(url.DatabaseName);
        }

        public static void RefreshClient(bool isExpand = false)
        {
        }

        ///// <summary>
        ///// 构造方法
        ///// </summary>
        ///// <param name="connStr">mongodb连接字符串</param>
        public MongoLogRepository(string connStr)
        {
            MongoUrl url = new MongoUrl(connStr);
            _client = new MongoClient(url);
            _database = _client.GetDatabase(url.DatabaseName);
        }
        #endregion

        #region 构造方法副本集 正式环境
        ///// <summary>
        ///// 构造方法副本集
        ///// </summary>
        //public MongoRepository()
        //{
        //    string connStr = ConfigurationManager.ConnectionStrings["mongo_address"].ConnectionString;
        //    string databaseName = ConfigurationManager.ConnectionStrings["mongo_edc"].ConnectionString;
        //    string replica = ConfigurationManager.ConnectionStrings["mongo_replica"].ConnectionString;
        //    var ips = connStr.Split(';');
        //    var servicesList = new List<MongoServerAddress>();
        //    foreach (var ip in ips)
        //    {
        //        var host = ip.Split(':')[0];
        //        var port = Convert.ToInt32(ip.Split(':')[1]);
        //        servicesList.Add(new MongoServerAddress(host, port));
        //    }
        //    _setting = new MongoClientSettings();
        //    _setting.ReplicaSetName = replica;
        //    _setting.Servers = servicesList;

        //    if (_client == null)
        //        _client = new MongoClient(_setting);

        //    if (_database == null)
        //        _database = _client.GetDatabase(databaseName);
        //}

        ///// <summary>
        ///// 构造方法
        ///// </summary>
        ///// <param name = "connStr" > mongodb连接字符串 </ param >
        //public MongoRepository(string connStr)
        //{
        //    MongoUrl url = new MongoUrl(connStr);
        //    _client = new MongoClient(url);
        //    _database = _client.GetDatabase(url.DatabaseName);
        //}
        #endregion

        #region 添加纪录

        #region 添加单条纪录（泛型）
        /// <summary>
        /// 添加纪录（泛型）
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="obj">对象</param>
        public void Add<T>(T obj)
        {
            string collectionName = GetCollectionName<T>();
            var collection = _database.GetCollection<T>(collectionName);
            collection.InsertOneAsync(obj);
        }
        #endregion

        #region 批量添加（泛型）
        /// <summary>
        /// 批量添加（泛型）
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="list">实体集合</param>
        public void AddMany<T>(List<T> list)
        {
            string collectionName = GetCollectionName<T>();
            var collection = _database.GetCollection<T>(collectionName);
            collection.InsertMany(list);
        }
        #endregion

        #region 添加单条纪录（bson文档）
        /// <summary>
        /// 添加纪录（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="bson">bson文档</param>
        public void Add(string collectionName, BsonDocument bson)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            collection.InsertOneAsync(bson);
        }
        #endregion

        #region 批量添加（bson文档）
        /// <summary>
        /// 批量添加（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="bson">bson文档集合</param>
        public void AddMany(string collectionName, List<BsonDocument> bson)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            collection.InsertManyAsync(bson);
        }
        #endregion

        #endregion

        #region 更新记录

        #region 更新一条记录（泛型）
        /// <summary>
        /// 更新一条记录（泛型）
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="obj">对象</param>
        public void Update<T>(T obj)
        {
            string tableName = string.Empty;
            string objId = string.Empty;
            var updateBson = ObjectToUpdateBson(obj, out tableName, out objId);
            if (!string.IsNullOrEmpty(tableName) && updateBson != null)
            {
                var filter = Builders<BsonDocument>.Filter.Eq("_id", objId);
                _database.GetCollection<BsonDocument>(tableName).FindOneAndUpdateAsync(filter, updateBson);
            }
        }

        public void Replace(string collectionName, FilterDefinition<BsonDocument> filter, BsonDocument document)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);

            collection.ReplaceOneAsync(filter, document);
        }


        /// <summary>
        /// 更新单条数据（泛型）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="collectionName">集合名称</param>
        /// <param name="filter">条件</param>
        /// <param name="update">更新内容</param>
        /// <returns>更新条数</returns>
        public long Update<T>(FilterDefinition<T> filter, UpdateDefinition<T> update, UpdateOptions updateOptions = null) where T : new()
        {
            string collectionName = GetCollectionName<T>();
            var collection = _database.GetCollection<T>(collectionName);
            var result = collection.UpdateOneAsync(filter, update, updateOptions).Result;
            return result.ModifiedCount;
        }


        public long Update<T>(string collectionName, FilterDefinition<T> filter, UpdateDefinition<T> update, UpdateOptions updateOptions = null) where T : new()
        {
            var collection = _database.GetCollection<T>(collectionName);
            var result = collection.UpdateOneAsync(filter, update, updateOptions).Result;
            return result.ModifiedCount;
        }
        #endregion

        #region 批量更新记录（泛型）
        /// <summary>
        /// 批量更新记录（泛型）
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="expre">更新表达式</param>
        /// <param name="updateDefinition">更新内容</param>
        public void UpdateMany<T>(Expression<Func<T, bool>> expre, UpdateDefinition<T> updateDefinition)
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            collection.UpdateManyAsync<T>(expre, updateDefinition);
        }
        #endregion

        #region 更新记录（bson文档）
        /// <summary>
        /// 更新一条记录（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">更新条件</param>
        /// <param name="updateDefinition">更新内容</param>
        public long Update(string collectionName, FilterDefinition<BsonDocument> filter, UpdateDefinition<BsonDocument> updateDefinition)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            //Task<BsonDocument> task = collection.FindOneAndUpdateAsync(filter, updateDefinition, new FindOneAndUpdateOptions<BsonDocument>() { IsUpsert = true });
            //BsonDocument result = task.Result;
            UpdateResult result = collection.UpdateOne(filter, updateDefinition, new UpdateOptions { IsUpsert = true });
            return result.ModifiedCount;
        }
        #endregion

        #region 更新记录（bson文档）
        /// <summary>
        /// 更新一条记录（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">更新条件</param>
        /// <param name="updateDefinition">更新内容</param>
        public void UpdateMany(string collectionName, FilterDefinition<BsonDocument> filter, UpdateDefinition<BsonDocument> updateDefinition)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);

            collection.UpdateManyAsync(filter, updateDefinition);
        }
        #endregion

        #endregion

        #region 删除记录

        #region 删除单条记录（泛型）
        /// <summary>
        /// 删除单条记录（泛型）
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="expre">条件表达式</param>
        public void Delete<T>(Expression<Func<T, bool>> expre)
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            collection.FindOneAndDeleteAsync<T>(expre);
        }

        public long Delete<T>(string collectionName, FilterDefinition<T> filter)
        {
            var collection = _database.GetCollection<T>(collectionName);

            return collection.DeleteOne(filter).DeletedCount;
        }

        #endregion

        #region 批量删除记录（泛型）
        /// <summary>
        /// 批量删除记录（泛型）
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="expre">条件表达式</param>
        public void DeleteMany<T>(Expression<Func<T, bool>> expre)
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            collection.DeleteManyAsync<T>(expre);
        }
        #endregion

        #region 删除单条记录(bson文档)
        /// <summary>
        /// 删除单条记录(bson文档)
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">删除条件</param>
        public void Delete(string collectionName, FilterDefinition<BsonDocument> filter)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            collection.FindOneAndDeleteAsync(filter);
        }
        #endregion

        #region 批量删除记录(bson文档)
        /// <summary>
        /// 批量删除记录(bson文档)
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">删除条件</param>
        public void DeleteMany(string collectionName, FilterDefinition<BsonDocument> filter)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            collection.DeleteMany(filter);
        }
        public void DeleteMany<T>(string collectionName, FilterDefinition<T> filter)
        {
            var collection = _database.GetCollection<T>(collectionName);
            collection.DeleteMany(filter);
        }
        #endregion

        #endregion

        #region 查询记录

        #region 根据条件查询（泛型）
        /// <summary>
        /// 根据条件查询（泛型）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="expre">查询条件</param>
        /// <returns></returns>
        public Task<List<T>> Query<T>(Expression<Func<T, bool>> expre) where T : BaseEntity
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            return collection.Find<T>(expre).ToListAsync();
        }
        #endregion

        #region 根据条件分页查询（泛型）
        /// <summary>
        /// 根据条件分页查询（泛型）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="expre">条件</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">没页大小</param>
        /// <returns></returns>
        public Task<List<T>> PagingQuery<T>(Expression<Func<T, bool>> expre, int pageIndex, int pageSize) where T : BaseEntity
        {
            int skip = (pageIndex - 1) * pageSize;

            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            return collection.Find<T>(expre).Skip(skip).Limit(pageSize).ToListAsync();
        }
        #endregion

        #region 根据条件分页，排序查询（泛型）
        /// <summary>
        /// 根据条件分页，排序查询（泛型）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="expre">查询条件</param>
        /// <param name="orderExpre">排序字段</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页大小</param>
        /// <param name="isAsc">是否正序</param>
        /// <returns></returns>
        public Task<List<T>> PagingQueryAndOrder<T>(Expression<Func<T, bool>> expre, Expression<Func<T, object>> sort, int pageIndex, int pageSize, bool isAsc) where T : BaseEntity
        {
            int skip = (pageIndex - 1) * pageSize;

            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            if (isAsc)
            {
                return collection.Find<T>(expre).SortBy(sort).Skip(skip).Limit(pageSize).ToListAsync();
            }
            else
            {
                return collection.Find<T>(expre).SortByDescending(sort).Skip(skip).Limit(pageSize).ToListAsync();
            }
        }


        /// <summary>
        /// 根据条件分页，排序查询（泛型）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="expre">查询条件</param>
        /// <param name="orderExpre">排序字段</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页大小</param>
        /// <param name="isAsc">是否正序</param>
        /// <returns></returns>
        public Task<List<T>> PagingQueryAndOrderWithoutBaseEntity<T>(Expression<Func<T, bool>> expre, Expression<Func<T, object>> sort, int pageIndex, int pageSize, bool isAsc)
        {
            int skip = (pageIndex - 1) * pageSize;

            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            if (isAsc)
            {
                return collection.Find<T>(expre).SortBy(sort).Skip(skip).Limit(pageSize).ToListAsync();
            }
            else
            {
                return collection.Find<T>(expre).SortByDescending(sort).Skip(skip).Limit(pageSize).ToListAsync();
            }
        }
        #endregion

        #region 查询记录条数（泛型）
        /// <summary>
        /// 查询记录条数（泛型）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="expre">查询条件</param>
        /// <returns></returns>
        public long GetCount<T>(Expression<Func<T, bool>> expre)
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            return collection.Count(expre);
        }
        #endregion

        #region 根据条件查询（bson文档）
        /// <summary>
        /// 根据条件查询（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">查询条件</param>
        /// <returns></returns>
        public Task<List<BsonDocument>> Query(string collectionName, FilterDefinition<BsonDocument> filter)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            return collection.Find(filter).ToListAsync();
        }

        public Task<BsonDocument> Query(string collectionName, FilterDefinition<BsonDocument> filter, ProjectionDefinition<BsonDocument> projection)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            return collection.Find(filter).Project(projection).SingleOrDefaultAsync();
        }

        public Task<List<BsonDocument>> Query(string collectionName, FilterDefinition<BsonDocument> filter, SortDefinition<BsonDocument> sort)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            return collection.Find(filter).Sort(sort).ToListAsync();
        }

        #endregion

        #region 根据条件分页查询（bson文档）
        /// <summary>
        /// 根据条件分页查询（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">查询条件</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页大小</param>
        /// <returns></returns>
        public Task<List<BsonDocument>> PagingQuery(string collectionName, FilterDefinition<BsonDocument> filter, int pageIndex, int pageSize)
        {
            int skip = (pageIndex - 1) * pageSize;

            var collection = _database.GetCollection<BsonDocument>(collectionName);

            return collection.Find(filter).Skip(skip).Limit(pageSize).ToListAsync();
        }
        #endregion

        #region 根据条件分页，排序查询（bson文档）
        /// <summary>
        /// 根据条件分页，排序查询（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">查询条件</param>
        /// <param name="sort">排序</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页大小</param>
        /// <param name="isAsc">是否正序</param>
        /// <returns></returns>
        public Task<List<BsonDocument>> PagingQueryAndOrder(string collectionName, FilterDefinition<BsonDocument> filter, SortDefinition<BsonDocument> sort, int pageIndex, int pageSize)
        {
            int skip = (pageIndex - 1) * pageSize;

            var collection = _database.GetCollection<BsonDocument>(collectionName);

            return collection.Find(filter).Sort(sort).Skip(skip).Limit(pageSize).ToListAsync();
        }
        #endregion

        #region 查询记录条数（bson文档）
        /// <summary>
        /// 查询记录条数（bson文档）
        /// </summary>
        /// <param name="collectionName">文档名称</param>
        /// <param name="filter">查询条件</param>
        /// <returns></returns>
        public long GetCount(string collectionName, FilterDefinition<BsonDocument> filter)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);

            return collection.Count(filter);
        }


        /// <summary>
        /// 查询记录条数（泛型）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="expre">查询条件</param>
        /// <returns></returns>
        public long GetCountWithoutBaseEntity<T>(Expression<Func<T, bool>> expre)
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            return collection.Count(expre);
        }
        #endregion


        /// <summary>
        /// 查询多条数据（泛型）,数据库中字段必须与实体对应
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="collectionName">collection名称</param>
        /// <param name="filter">查询表达式</param>
        /// <param name="projection">需要查询的字段，NULL默认查询全部</param>
        /// <param name="sort">排序表达式</param>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页大小</param>
        /// <param name="readCount">总条数</param>
        /// <param name="usePage">是否分页</param>
        /// <returns></returns>
        public Task<List<T>> Read<T>(FilterDefinition<T> filter, ProjectionDefinition<T, T> projection, SortDefinition<T> sort, int? pageIndex, int? pageSize, out int readCount, bool usePage = true) where T : new()
        {
            string collectionName = GetCollectionName<T>();

            if (filter == null)
                filter = Builders<T>.Filter.Empty;

            var collection = _database.GetCollection<T>(collectionName);

            if (usePage)
            {
                readCount = (int)collection.CountAsync(filter).Result;

                pageIndex = pageIndex ?? 1;

                pageSize = pageSize ?? 10;

                int skip = (pageIndex.Value - 1) * pageSize.Value;

                if (projection == null)
                {
                    if (sort == null)
                        return collection.Find(filter).Skip(skip).Limit(pageSize).ToListAsync();
                    else
                        return collection.Find(filter).Sort(sort).Skip(skip).Limit(pageSize).ToListAsync();
                }
                else
                {
                    if (sort == null)
                        return collection.Find(filter).Project(projection).Skip(skip).Limit(pageSize).ToListAsync();
                    else
                        return collection.Find(filter).Project(projection).Sort(sort).Skip(skip).Limit(pageSize).ToListAsync();
                }
            }
            else
            {
                readCount = 0;

                if (sort == null)
                    return collection.Find(filter).ToListAsync();
                else
                    return collection.Find(filter).Sort(sort).ToListAsync();
            }
        }

        #endregion


        #region MapReduce
        public List<T> MepReduce<T>(BsonJavaScript map, BsonJavaScript reduce) where T : BaseEntity
        {
            string collectionName = GetCollectionName<T>();

            var collection = _database.GetCollection<T>(collectionName);

            return collection.MapReduce<T>(map, reduce).ToList<T>();
        }


        public List<T> MepReduce<T>(BsonJavaScript map, BsonJavaScript reduce, string collectionName)
        {
            var collection = _database.GetCollection<T>(collectionName);
            return collection.MapReduce<T>(map, reduce).ToList<T>();
        }


        public List<TR> MepReduce<TR>(BsonJavaScript map, BsonJavaScript reduce, BsonJavaScript finalize, string collectionName)
        {
            var collection = _database.GetCollection<TR>(collectionName);
            var options = new MapReduceOptions<TR, TR>();
            options.Finalize = finalize;
            return collection.MapReduce<TR>(map, reduce, options).ToList<TR>();
        }

        public List<TR> MepReduce<TR, TD>(BsonJavaScript map, BsonJavaScript reduce, string collectionName, BsonJavaScript finalize = null, FilterDefinition<TD> filter = null)
        {
            var collection = _database.GetCollection<TD>(collectionName);
            var options = new MapReduceOptions<TD, TR>();
            options.Finalize = finalize;
            options.Filter = filter;
            return collection.MapReduce<TR>(map, reduce, options).ToList<TR>();
        }

        #endregion
        #region Common

        #region 获取实体类对应的集合名称
        /// <summary>
        /// 获取实体类对应的集合名称
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <returns></returns>
        public static string GetCollectionName<T>()
        {
            Type t = typeof(T);
            return t.Name.ToLower();
        }
        #endregion

        #region 对象转BSON文档
        /// <summary>
        /// 对象转BSON
        /// </summary>
        /// <param name="sourceObj">要转换的对象</param>
        /// <param name="objTypeName">实体对应的名称</param>
        /// <returns>bson文档</returns>
        public static BsonDocument ObjectToBson(object sourceObj, out string objTypeName)
        {
            var properties = sourceObj.GetType().GetProperties();
            objTypeName = sourceObj.GetType().Name.ToLower();
            if (properties != null && properties.Count() > 0)
            {
                BsonDocument bson = new BsonDocument();
                foreach (var p in properties)
                {
                    if (p != null && p.CanWrite)
                    {
                        string name = p.Name;
                        var value = p.GetValue(sourceObj, null);
                        bson.Add(new BsonElement(name, BsonValue.Create(value)));
                    }
                    else
                    {
                        continue;
                    }
                }
                return bson;
            }
            else
            {
                return null;
            }
        }
        #endregion

        #region 对象集合转BSON文档集合
        /// <summary>
        /// 对象转BSON
        /// </summary>
        /// <param name="sourceList">要转换的对象集合</param>
        /// <param name="objTypeName">实体对应的名称</param>
        /// <returns>bson文档</returns>
        public static List<BsonDocument> ObjectListToBsonList<T>(List<T> sourceList, out string objTypeName)
        {
            List<BsonDocument> bsonList = new List<BsonDocument>();
            if (sourceList != null && sourceList.Count > 0)
            {
                objTypeName = sourceList.FirstOrDefault().GetType().Name.ToLower();
                foreach (var item in sourceList)
                {
                    var properties = item.GetType().GetProperties();

                    if (properties != null && properties.Count() > 0)
                    {
                        BsonDocument bson = new BsonDocument();
                        foreach (var p in properties)
                        {
                            if (p != null && p.CanWrite)
                            {
                                string name = p.Name;
                                var value = p.GetValue(item, null);
                                bson.Add(new BsonElement(name, BsonValue.Create(value)));
                            }
                            else
                            {
                                continue;
                            }
                        }
                        bsonList.Add(bson);
                    }
                    else
                    {
                        continue;
                    }
                }
            }
            else
            {
                objTypeName = string.Empty;
            }
            return bsonList;
        }
        #endregion

        #region 对象转update对象
        /// <summary>
        /// 对象转update对象
        /// </summary>
        /// <param name="sourceObj">要转换的对象</param>
        /// <param name="objTypeName">对象类型</param>
        /// <param name="objId">objectID</param>
        /// <returns></returns>
        public static UpdateDocument ObjectToUpdateBson(object sourceObj, out string objTypeName, out string objId)
        {
            var properties = sourceObj.GetType().GetProperties();
            objTypeName = sourceObj.GetType().Name.ToLower();
            objId = string.Empty;
            if (properties != null && properties.Count() > 0)
            {
                UpdateDocument bson = new UpdateDocument();
                foreach (var p in properties)
                {
                    if (p != null && p.CanWrite)
                    {
                        string name = p.Name;
                        var value = p.GetValue(sourceObj, null);
                        if (name == "_id")
                        {
                            objId = value.ToString();
                        }
                        bson.Add(new BsonElement(name, BsonValue.Create(value)));
                    }
                    else
                    {
                        continue;
                    }
                }
                return bson;
            }
            else
            {
                return null;
            }
        }
        #endregion

        #region 对象转update对象
        /// <summary>
        /// 对象转update对象
        /// </summary>
        /// <param name="sourceObjList">要转换的对象集合</param>
        /// <param name="objTypeName">对象类型</param>
        /// <returns></returns>
        public static List<UpdateDocument> ObjectListToUpdateBsonList<T>(List<T> sourceObjList, out string objTypeName)
        {
            List<UpdateDocument> list = new List<UpdateDocument>();
            if (sourceObjList != null && sourceObjList.Count > 0)
            {
                objTypeName = sourceObjList.FirstOrDefault().GetType().Name.ToLower();
                foreach (var item in sourceObjList)
                {
                    var properties = item.GetType().GetProperties();

                    if (properties != null && properties.Count() > 0)
                    {
                        UpdateDocument bson = new UpdateDocument();
                        foreach (var p in properties)
                        {
                            if (p != null && p.CanWrite)
                            {
                                string name = p.Name;
                                var value = p.GetValue(item, null);
                                bson.Add(new BsonElement(name, BsonValue.Create(value)));
                            }
                            else
                            {
                                continue;
                            }
                        }
                        list.Add(bson);
                    }
                    else
                    {
                        continue;
                    }
                }
            }
            else
            {
                objTypeName = string.Empty;
            }
            return list;
        }
        #endregion

        #region JSON字符串转Bson文档
        /// <summary>
        /// JSON字符串转Bson文档
        /// </summary>
        /// <param name="json">要转换的JSON字符串</param>
        /// <returns>bson文档,失败返回NULL</returns>
        public static BsonDocument JsonToBson(string json)
        {
            try
            {
                return BsonDocument.Parse(json);
            }
            catch (Exception ex)
            {
                return null;
            }
        }
        #endregion

        #endregion

        #region 聚合
        /// <summary>
        /// 聚合
        /// </summary>
        /// <param name="collectionName">集合名称</param>
        /// <param name="pipeline">管道信息</param>
        /// <returns></returns>
        public List<BsonDocument> Aggregate(string collectionName, PipelineStagePipelineDefinition<BsonDocument, BsonDocument> pipeline)
        {
            var collection = _database.GetCollection<BsonDocument>(collectionName);
            var cursor = collection.Aggregate(pipeline);
            return cursor.ToList();
        }
        #endregion
    }
}
